unit Image;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, Jpeg, StdCtrls, Buttons;

type
  TFImage = class(TForm)
    ViewPort: TPaintBox;
    Panel1: TPanel;
    Label1: TLabel;
    Label2: TLabel;
    lblFileName: TLabel;
    lblDateTime: TLabel;
    btnFirst: TSpeedButton;
    btnPrev: TSpeedButton;
    btnNext: TSpeedButton;
    btnLast: TSpeedButton;
    btnZoomIn: TSpeedButton;
    bntZoomOut: TSpeedButton;
    btnTerminate: TSpeedButton;
    Label3: TLabel;
    lblSize: TLabel;
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ViewPortPaint(Sender: TObject);
    procedure btnTerminateClick(Sender: TObject);
    procedure btnFirstClick(Sender: TObject);
    procedure btnPrevClick(Sender: TObject);
    procedure btnNextClick(Sender: TObject);
    procedure btnLastClick(Sender: TObject);
    procedure BtnZoomInClick(Sender: TObject);
    procedure btnZoomOutClick(Sender: TObject);
  private
    FList: TList;
    FImageIndex: integer;
    FDisplayImage: TJPEGImage;
    FRatio: double;
    FImageRect: TRect;
    FViewPortSize: TSize;
    procedure SetImageIndex(AImageIndex: integer);
    procedure SetRatio(ARatio: double);
    procedure CreateDisplayImage;
    property ImageIndex: integer read FImageIndex write SetImageIndex;
    property Ratio: double read FRatio write SetRatio;
    { Private declarations }
  public
    procedure Execute(ViewList: TList; ActiveIdx: integer);
    { Public declarations }
  end;

var
  FImage: TFImage;

implementation

uses
  Lister, Utils;

{$R *.dfm}

const
  SErrorCapt = 'Chyba';
  SLoadError = 'Soubor nelze nacist...';

procedure TFImage.Execute(ViewList: TList; ActiveIdx: integer);
begin
  FViewPortSize.cx := Screen.Width;
  FViewPortSize.cy := Screen.Height - Panel1.Height;

  FList := ViewList;
  ImageIndex := ActiveIdx;

  WindowState := wsMaximized;
  ShowModal;
end;

procedure TFImage.FormClose(Sender: TObject; var Action: TCloseAction);
var
  i: integer;

begin
  for i := 0 to FList.Count - 1 do
    Dispose(FList[i]);
  FList.Clear;
  FList.Free;
end;

procedure TFImage.FormCreate(Sender: TObject);
begin
  FDisplayImage := TJPEGImage.Create;
end;

procedure TFImage.FormDestroy(Sender: TObject);
begin
  FDisplayImage.Free;
end;

procedure TFImage.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  case Key of
    VK_RETURN: Close;
    VK_NEXT: btnNextClick(nil);
    VK_PRIOR: btnPrevClick(nil);
    VK_DIVIDE: Ratio := 0;
    VK_MULTIPLY: Ratio := 1;
    VK_ADD: BtnZoomInClick(nil);
    VK_SUBTRACT: btnZoomOutClick(nil);
    VK_HOME: btnFirstClick(nil);
    VK_END: btnLastClick(nil);
    VK_ESCAPE: btnTerminateClick(nil);
  end;
end;

procedure TFImage.SetImageIndex(AImageIndex: integer);
var
  Info: PNodeInfo;
  Size: integer;
  Time: TDateTime;

begin
  if FList = nil then
    exit;
  if (AImageIndex < 0) or (AImageIndex > FList.Count - 1) then
    exit;
  FImageIndex := AImageIndex;

  Info := FList[FImageIndex];
  Size := GetFileSize(Info^.RealPath);
  Time := GetFileDateTime(Info^.RealPath);

  lblFileName.Caption := Format('%s (%d byte) [%d/%d]',
    [Info^.RealPath, Size, FImageIndex+1, FList.Count]) ;
  lblDateTime.Caption := FormatDateTime('hh:nn:ss dd.mm.yyyy', Time);

  FRatio := 0;
  CreateDisplayImage;
end;

procedure TFImage.SetRatio(ARatio: Double);
begin
  FRatio := ARatio;
  CreateDisplayImage;
end;

procedure TFImage.ViewPortPaint(Sender: TObject);
begin
  ViewPort.Canvas.StretchDraw(FImageRect, FDisplayImage);
end;

procedure TFImage.btnZoomOutClick(Sender: TObject);
begin
  Ratio := Ratio / 2;
end;

procedure TFImage.btnFirstClick(Sender: TObject);
begin
  ImageIndex := 0;
end;

procedure TFImage.btnLastClick(Sender: TObject);
begin
  ImageIndex := FList.Count - 1;
end;

procedure TFImage.btnNextClick(Sender: TObject);
begin
  ImageIndex := ImageIndex + 1;
end;

procedure TFImage.btnPrevClick(Sender: TObject);
begin
  ImageIndex := ImageIndex - 1;
end;

procedure TFImage.btnTerminateClick(Sender: TObject);
begin
  Application.Terminate;
end;

procedure TFImage.BtnZoomInClick(Sender: TObject);
begin
  Ratio := Ratio * 2;
end;

procedure TFImage.CreateDisplayImage;
var
  Info: PNodeInfo;
  RealSize: TSize;
  StretchSize: TSize;
  ImageOffset: TPoint;
  rw, rh: double;
  CenterOnly: boolean;

begin
  Info := FList[FImageIndex];
  try
    FDisplayImage.LoadFromFile(Info^.RealPath);
  except
    Application.MessageBox(SLoadError, SErrorCapt, MB_ICONERROR or MB_OK);
    exit;
  end;

  RealSize.cx := FDisplayImage.Width;
  RealSize.cy := FDisplayImage.Height;

  CenterOnly := FRatio <> 0;
  if FRatio = 0 then
  begin //fit
    rw := FViewPortSize.cx / RealSize.cx;
    rh := FViewPortSize.cy / RealSize.cy;
    if rw < rh then
      FRatio := rw
    else
      FRatio := rh;
  end;

  StretchSize.cx := Round(RealSize.cx * FRatio);
  StretchSize.cy := Round(RealSize.cy * FRatio);

  if CenterOnly then
  begin
    ImageOffset.X := (FViewPortSize.cx - StretchSize.cx) div 2;
    ImageOffset.Y := (FViewPortSize.cy - StretchSize.cy) div 2;
  end
  else
  begin
    if rw < rh then
    begin
      ImageOffset.X := 0;
      ImageOffset.Y := Abs(StretchSize.cy - FViewPortSize.cy) div 2;
    end
    else
    begin
      ImageOffset.X := Abs(StretchSize.cx - FViewPortSize.cx) div 2;
      ImageOffset.Y := 0;
    end;
  end;

  lblSize.Caption := Format('%d x %d pixel (%.0f %%)', [RealSize.cx, RealSize.cy, Ratio * 100]);
  FImageRect := Bounds(ImageOffset.x, ImageOffset.y, StretchSize.cx, StretchSize.cy);
  ViewPort.Invalidate;
end;

end.
