unit Lister;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, ComCtrls, StdCtrls, ImgList, JPEG;

type
  PNodeInfo = ^TNodeInfo;
  TNodeInfo = record
    RealPath: string;
  end;

  TFLister = class(TForm)
    TreeView1: TTreeView;
    ListView1: TListView;
    Splitter1: TSplitter;
    Panel1: TPanel;
    Label1: TLabel;
    Label2: TLabel;
    PaintBox1: TPaintBox;
    lblFileName: TLabel;
    lblFileDate: TLabel;
    procedure TreeView1Expanding(Sender: TObject; Node: TTreeNode;
      var AllowExpansion: Boolean);
    procedure TreeView1Change(Sender: TObject; Node: TTreeNode);
    procedure ListView1KeyPress(Sender: TObject; var Key: Char);
    procedure FormCreate(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure ListView1Change(Sender: TObject; Item: TListItem;
      Change: TItemChange);
    procedure FormDestroy(Sender: TObject);
    procedure PaintBox1Paint(Sender: TObject);
  private
    FCurrentThumb: TBitmap;
    function HasSubdirs(Dir: string): boolean;
    function IsImage(FileName: string): boolean;
    function BuildViewList: TList;
    procedure FillTree(Root: string; RootNode: TTreeNode);
    procedure FillList(Dir: string);
    { Private declarations }
  public
    procedure NavigateTo(Dir: string);
    { Public declarations }
  end;

var
  FLister: TFLister;

implementation

uses Image, Utils;

{$R *.dfm}

function TFLister.IsImage(FileName: string): boolean;
var
  Ext: string;

begin
  Ext := UpperCase(ExtractFileExt(FileName));
  result := (Ext = '.JPG') or (Ext = '.JPEG');
end;

procedure TFLister.ListView1Change(Sender: TObject; Item: TListItem;
  Change: TItemChange);
var
  Info: PNodeInfo;
  Size: integer;
  DateTime: TDateTime;
  JPEG: TJPEGImage;

begin
  if Item = nil then
    exit;
  Info := Item.Data;
  if Info = nil then
    exit;

  DateTime := GetFileDateTime(Info^.RealPath);
  Size := GetFileSize(Info^.RealPath);

  lblFileName.Caption := Format('%s (%d byte)',
    [Info^.RealPath, Size]);
  lblFileDate.Caption := FormatDateTime('hh:nn:ss dd.mm.yyyy', DateTime);
end;

procedure TFLister.ListView1KeyPress(Sender: TObject; var Key: Char);
var
  ViewList: TList;

begin
  if (Key = #13) and (ListView1.ItemIndex <> -1) then
  begin
    ViewList := BuildViewList;
    FImage.Execute(ViewList, ListView1.ItemIndex);
  end;
end;

procedure TFLister.TreeView1Change(Sender: TObject; Node: TTreeNode);
var
  i: integer;
  Info: PNodeInfo;

begin
  try
    ListView1.Items.BeginUpdate;
    Screen.Cursor := crHourGlass;

    for i := 0 to ListView1.Items.Count - 1 do
      Dispose(ListView1.Items[i].Data);
    ListView1.Items.Clear;

    Info := Node.Data;
    if Info <> nil then
      FillList(Info.RealPath);
  finally
    ListView1.Items.EndUpdate;
    Screen.Cursor := crDefault;
  end;
end;

procedure TFLister.TreeView1Expanding(Sender: TObject; Node: TTreeNode;
  var AllowExpansion: Boolean);
var
  Info: PNodeInfo;

begin
  Node.DeleteChildren;
  Info := Node.Data;
  if Info <> nil then
    FillTree(Info^.RealPath, Node);
end;

function TFLister.HasSubdirs(Dir: string): boolean;
var
  Search: TSearchRec;
  Err: integer;

begin
  result := false;
  Dir := IncludeTrailingPathDelimiter(Dir);
  Err := FindFirst(Dir + '*.*', faAnyFile, Search);
  while Err = 0 do
  begin
    if ((Search.Attr and faDirectory) <> 0) then
    begin
      if (Search.Name <> '..') and (Search.Name <> '.') then
      begin
        result := true;
        break;
      end;
    end;
    Err := FindNext(Search);
  end;
end;

procedure TFLister.FillTree(Root: string; RootNode: TTreeNode);
var
  r: TSearchRec;
  e: integer;
  DirNode: TTreeNode;
  Info: PNodeInfo;

begin
  Root := IncludeTrailingPathDelimiter(Root);

  e := FindFirst(Root + '*.*', faAnyFile, r);
  while e = 0 do
  begin
    if (r.Attr and faDirectory) <> 0 then
      if (r.Name <> '.') and (r.Name <> '..') then
      begin
        DirNode := TreeView1.Items.AddChild(RootNode, r.Name);
        if HasSubdirs(Root + r.Name) then
          TreeView1.Items.AddChild(DirNode, 'dummy');

        New(Info);
        Info^.RealPath := Root + r.Name;
        DirNode.Data := Info;
      end;
    e := FindNext(r);
  end;
end;

procedure TFLister.FormCreate(Sender: TObject);
var
  Drives: cardinal;
  Drive: char;
  Path: string;
  NewNode: TTreeNode;
  Info: PNodeInfo;

begin
  SetErrorMode(SEM_FAILCRITICALERRORS);

  FCurrentThumb := TBitmap.Create;
  with FCurrentThumb do
  begin
    Height := PaintBox1.Height;
    Width := PaintBox1.Width;
  end;

  Drives := GetLogicalDrives;
  for Drive := 'A' to 'Z' do
  begin
    if (Drives and $01) <> 0 then
    begin
      Path := Drive + ':';
      New(Info);
      Info^.RealPath := Path;

      NewNode := TreeView1.Items.AddChild(nil, Path);
      NewNode.Data := Info;
      FillTree(Path, NewNode);
    end;
    Drives := Drives shr 1;
  end;
end;

procedure TFLister.FormDestroy(Sender: TObject);
begin
  FCurrentThumb.Free;
end;

procedure TFLister.Button1Click(Sender: TObject);
begin
  NavigateTo(GetCurrentDir);
end;

procedure TFLister.FillList(Dir: string);
var
  Search: TSearchRec;
  Err: integer;
  Info: PNodeInfo;
  Thumb: TBitmap;

begin
  Dir := IncludeTrailingPathDelimiter(Dir);
  Err := FindFirst(Dir + '*.*', faAnyFile - faDirectory, Search);
  while Err = 0 do
  begin
    if IsImage(Dir + Search.Name) then
      with ListView1.Items.Add do
      begin
        New(Info);
        Info^.RealPath := Dir + Search.Name;
        Data := Info;
        Caption := Search.Name;
      end;
    Err := FindNext(Search);
  end;
end;

function TFLister.BuildViewList: TList;
var
  i: integer;
  Data: PNodeInfo;

begin
  result := TList.Create;
  for i := 0 to ListView1.Items.Count - 1 do
  begin
    New(Data);
    Data^ := PNodeInfo(ListView1.Items[i].Data)^;
    result.Add(Data);
  end;
end;

procedure TFLister.NavigateTo(Dir: string);
begin
end;

procedure TFLister.PaintBox1Paint(Sender: TObject);
begin
  PaintBox1.Canvas.Draw(0, 0, FCurrentThumb);
end;

end.
